﻿//This code is used for batch converting the recorded video data (in .dat format) to avi. 
//It assumes that each .dat file is an 8 bit greyscale video, saved in its own folder along with a metadata.csv file.
//Insert the top level folder path that the video folders are saved in to the box in the form and click the button
//Avi files will be added to the video folders

//Required libraries:
//	Basler Pylon API (https://www.baslerweb.com/en/products/software/basler-pylon-camera-software-suite/)
//	Serialization
//	AviFile

//Please email Benjamin Smith (bsmith@rvc.ac.uk) to report bugs or with any questions
//Code copyright Benjamin Smith 2019
//This work is licensed under a Creative Commons Attribution 4.0 International License.

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Basler.Pylon;
using System.Runtime.Serialization.Formatters.Binary;
using System.IO;
using AviFile;

namespace convert_to_avi
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        public static object Deserialize(string path)
        {
            using (Stream stream = File.Open(path, FileMode.Open))
            {
                BinaryFormatter bformatter = new BinaryFormatter();
                return bformatter.Deserialize(stream);
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            string vpath = this.filepath.Text;

            List<string> dirs = new List<string>(Directory.EnumerateDirectories(vpath));

            foreach (var dir in dirs)
            {
                string subvpath = dir.Substring(dir.LastIndexOf("\\") + 1);
                Console.WriteLine(subvpath);

                byte[][] vid_buffer = (byte[][])Deserialize(vpath + "/" + subvpath + "/imgpxdata.dat");
                Console.WriteLine("Data loaded");

                int width = 1280, height = 640; //dimensions of each video frame - change as appropriate

                AviManager aviManager = new AviManager(vpath + "/" + subvpath + "/video_comp1.avi", false);

                byte[] vid0 = vid_buffer[0];
                byte[] grayscale_image0 = vid0.Skip(0).Take(819200).ToArray(); //make sure there are the correct number of pixels per image- 1280 x 640 = 819200
                System.Drawing.Bitmap bitmap0 = new System.Drawing.Bitmap(width, height);
                int x0 = 0;
                int y0 = 0;
                foreach (int i in grayscale_image0)
                {
                    bitmap0.SetPixel(x0, y0, System.Drawing.Color.FromArgb(i, i, i));
                    x0++;
                    if (x0 >= width)
                    {
                        x0 = 0;
                        y0++;
                    }
                }
                Console.WriteLine("Adding frame 0");
                VideoStream aviStream = aviManager.AddVideoStream(false, 2, bitmap0); //new video and add first frame
                bitmap0.Dispose();


                int c = vid_buffer.Length;
                progressBar1.Minimum = 1;
                progressBar1.Maximum = c;

                for (int j = 1; j < c; ++j)
                {
                    byte[] vid1 = vid_buffer[j];
                    byte[] grayscale_image = vid1.Skip(0).Take(819200).ToArray(); //make sure there are the correct number of pixels
                    System.Drawing.Bitmap bitmap = new System.Drawing.Bitmap(width, height);
                    int x = 0;
                    int y = 0;
                    foreach (int i in grayscale_image)
                    {
                        bitmap.SetPixel(x, y, System.Drawing.Color.FromArgb(i, i, i));//create frame image from the pixel data
                        x++;
                        if (x >= width)
                        {
                            x = 0;
                            y++;
                        }
                    }

                    label2.Text = "Adding frame " + j.ToString() + " of " + c.ToString();
                    label2.Refresh();
                    aviStream.AddFrame(bitmap);//add the frame to the video
                    bitmap.Dispose();
                    progressBar1.Value = j;

                }
                aviManager.Close();
                Console.WriteLine("Video complete");
            }
                System.Windows.Forms.Application.Exit();
            
        }

        private void label1_Click(object sender, EventArgs e)
        {

        }
    }
}
